@@ -1,6 +1,4 @@
-"""
-Representing a GasOil object
-"""
+""" Representing a GasOil object """
 from __future__ import division, absolute_import
 from __future__ import print_function
 
@@ -274,6 +272,10 @@ class GasOil(object):
         if krgcolname in dframe:
             if not (dframe[krgcolname].diff().dropna() > -epsilon).all():
                 raise ValueError("Incoming krg not increasing")
+            if dframe[krgcolname].max() > 1.0:
+                raise ValueError("krg is above 1 in incoming table")
+            if dframe[krgcolname].min() < 0.0:
+                raise ValueError("krg is below 0 in incoming table")
             pchip = PchipInterpolator(
                 dframe[sgcolname].astype(float), dframe[krgcolname].astype(float)
             )
@@ -281,16 +283,22 @@ class GasOil(object):
             self.table["krg"] = pchip(self.table.sg, extrapolate=False)
             self.table["krg"].fillna(method="ffill", inplace=True)
             self.table["krg"].fillna(method="bfill", inplace=True)
+            self.table["krg"].clip(lower=0.0, upper=1.0, inplace=True)
             self.krgcomment = "-- krg from tabular input" + krgcomment + "\n"
         if krogcolname in dframe:
             if not (dframe[krogcolname].diff().dropna() < epsilon).all():
                 raise ValueError("Incoming krogcolname not decreasing")
+            if dframe[krogcolname].max() > 1.0:
+                raise ValueError("krog is above 1 in incoming table")
+            if dframe[krogcolname].min() < 0.0:
+                raise ValueError("krog is below 0 in incoming table")
             pchip = PchipInterpolator(
                 dframe[sgcolname].astype(float), dframe[krogcolname].astype(float)
             )
             self.table["krog"] = pchip(self.table.sg, extrapolate=False)
             self.table["krog"].fillna(method="ffill", inplace=True)
             self.table["krog"].fillna(method="bfill", inplace=True)
+            self.table["krog"].clip(lower=0.0, upper=1.0, inplace=True)
             self.krogcomment = "-- krog from tabular input" + krogcomment + "\n"
         if pccolname in dframe:
             # Incoming dataframe must cover the range:
@@ -307,10 +315,10 @@ class GasOil(object):
                 )
             dframe = dframe.replace([np.inf, -np.inf], np.nan)
             dframe.dropna(subset=[pccolname], how="all", inplace=True)
-            # If nonzero, then it must be decreasing:
+            # If nonzero, then it must be increasing:
             if dframe[pccolname].abs().sum() > 0:
-                if not (dframe[pccolname].diff().dropna() < 0.0).all():
-                    raise ValueError("Incoming pc not decreasing")
+                if not (dframe[pccolname].diff().dropna() > 0.0).all():
+                    raise ValueError("Incoming pc not increasing")
             pchip = PchipInterpolator(
                 dframe[sgcolname].astype(float), dframe[pccolname].astype(float)
             )
@@ -352,7 +360,9 @@ class GasOil(object):
                 krgmax = 1
             tmp = pd.DataFrame(self.table[["sg"]])
             tmp["sgendnorm"] = (tmp["sg"] - (1 - (self.sorg + self.swl))) / (self.sorg)
-            tmp["krg"] = tmp["sgendnorm"] * krgmax + (1 - tmp["sgendnorm"]) * krgend
+            tmp["krg"] = (
+                tmp["sgendnorm"] * krgmax + (1 - tmp["sgendnorm"]) * krgend
+            ).clip(lower=0.0, upper=1.0)
             self.table.loc[
                 self.table.sg >= (1 - (self.sorg + self.swl + epsilon)), "krg"
             ] = tmp.loc[tmp.sg >= (1 - (self.sorg + self.swl + epsilon)), "krg"]
@@ -737,8 +747,10 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += self.table[["sg", "krg", "krog", "pc"]].to_csv(
-            sep=" ", float_format="%1.7f", header=None, index=False
+        string += utils.df2str(
+            self.table[["sg", "krg", "krog", "pc"]],
+            monotone_column="pc",
+            monotone_direction="inc",
         )
         string += "/\n"
         return string
@@ -772,6 +784,8 @@ class GasOil(object):
                     map(int, list(map(round, slgof["sl"] * SWINTEGERS)))
                 )
                 slgof.drop_duplicates("slint", inplace=True)
+                # Delete the temporary column:
+                slgof.drop(labels="slint", axis="columns", inplace=True)
             else:
                 # Give up repairing the table:
                 logger.critical(
@@ -824,8 +838,8 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += self.slgof_df().to_csv(
-            sep=" ", float_format="%1.7f", header=None, index=False
+        string += utils.df2str(
+            self.slgof_df(), monotone_column="pc", monotone_direction="dec"
         )
         string += "/\n"
         return string
@@ -870,8 +884,10 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += self.table[["sg", "krg", "pc"]].to_csv(
-            sep=" ", float_format="%1.7f", header=None, index=False
+        string += utils.df2str(
+            self.table[["sg", "krg", "pc"]],
+            monotone_column="pc",
+            monotone_direction="inc",
         )
         string += "/\n"
         return string
@@ -917,8 +933,10 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += self.table[["sg", "krg", "krog", "pc"]].to_csv(
-            sep=" ", float_format="%1.7f", header=None, index=False
+        string += utils.df2str(
+            self.table[["sg", "krg", "krog", "pc"]],
+            monotone_column="pc",
+            monotone_direction="inc",
         )
         return string
 
@@ -929,6 +947,7 @@ class GasOil(object):
         alpha=1,
         linewidth=1,
         linestyle="-",
+        marker=None,
         label=None,
         logyscale=False,
     ):
@@ -959,6 +978,7 @@ class GasOil(object):
             label=label,
             linewidth=linewidth,
             linestyle=linestyle,
+            marker=marker,
         )
         self.table.plot(
             ax=useax,
@@ -970,6 +990,7 @@ class GasOil(object):
             label=None,
             linewidth=linewidth,
             linestyle=linestyle,
+            marker=marker,
         )
         if mpl_ax is None:
             plt.show()
