@@ -17,6 +17,70 @@ logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 
+def df2str(
+    dframe,
+    digits=7,
+    roundlevel=9,
+    header=False,
+    monotone_column=None,
+    monotone_direction=None,
+):
+    """
+    Make a string representation of a dataframe with
+    proper rounding.
+
+    This is used to print the tables in the SWOF/SGOF include files,
+    explicit rounding is necessary to avoid monotonicity errors
+    from truncation. Examples in test code.
+
+    Capillary pressure must be strictly monotone if nonzero, and if a
+    column name is provided, the string representation of that column is
+    ensured to be strictly monotone decreasing
+
+    Args:
+        dframe (pd.DataFrame): A dataframe to print, all columns
+            are included
+        digits (int): Number of digits used in floating point format f.ex ".7f"
+            It is not recommended to deviate from the default 7 uncritically
+            for pyscal output, other code have to be tuned to ensure
+            numerical robustness to the deviation.
+        roundlevel (int): To how many digits should we round prior to print.
+            Recommended to be > digits + 1, see test code.
+        header (bool): If the dataframe column header should be included
+        monotone_column: column name for which strict monotonicity
+            must be preserved in output. Only one column can be fixed.
+        monotone_direction: Direction of monotonicity, increasing or decreasing,
+            allowed values are '-1', '1', 'inc' or 'dec'
+    """
+    float_format = "%1." + str(digits) + "f"
+
+    if monotone_direction is not None:
+        if monotone_direction == 1 or monotone_direction == "inc":
+            sign = 1
+        elif monotone_direction == -1 or monotone_direction == "dec":
+            sign = -1
+        else:
+            raise ValueError("Unknown monotone_direction: " + str(monotone_direction))
+    else:
+        if monotone_column is not None:
+            sign = -1  # Default is decreasing monotonocity
+
+    if monotone_column is not None and dframe[monotone_column].abs().sum() > 0:
+        dframe = dframe.copy()
+        constants = dframe[monotone_column].round(digits).diff() == 0.0
+        while constants.any():
+            # Substract the smallest value that we can represent where
+            # it is needed:
+            dframe.loc[constants, monotone_column] = (
+                dframe.loc[constants, monotone_column] + sign / 10.0 ** digits
+            )
+            constants = dframe[monotone_column].round(digits).diff() == 0.0
+
+    return dframe.round(roundlevel).to_csv(
+        sep=" ", float_format=float_format, header=header, index=False
+    )
+
+
 def estimate_diffjumppoint(table, xcol=None, ycol=None, side="right"):
     """Estimate the point where the y-data jumps from being linear
     in x to being nonlinear, or where it shift from one linear domain
@@ -121,9 +185,12 @@ def normalize_nonlinpart_wo(curve):
     # The internal dataframe might contain normalized
     # saturation values, but we do not want to assume they
     # are there or even correct, therefore we effectively
-    # recalculate them (here using lambda functions)
-    sw_fn = lambda swn: curve.swcr + swn * (1.0 - curve.swcr - curve.sorw)
-    krw_fn = lambda swn: krw_interp(sw_fn(swn))
+    # recalculate them
+    def sw_fn(swn):
+        return curve.swcr + swn * (1.0 - curve.swcr - curve.sorw)
+
+    def krw_fn(swn):
+        return krw_interp(sw_fn(swn))
 
     kro_interp = interp1d(
         1.0 - curve.table["sw"],
@@ -132,8 +199,12 @@ def normalize_nonlinpart_wo(curve):
         bounds_error=False,
         fill_value=(0.0, curve.table["krow"].max()),
     )
-    so_fn = lambda son: curve.sorw + son * (1.0 - curve.sorw - curve.swcr)
-    kro_fn = lambda son: kro_interp(so_fn(son))
+
+    def so_fn(son):
+        return curve.sorw + son * (1.0 - curve.sorw - curve.swcr)
+
+    def kro_fn(son):
+        return kro_interp(so_fn(son))
 
     return (krw_fn, kro_fn)
 
@@ -160,7 +231,7 @@ def normalize_nonlinpart_go(curve):
         curve (GasOil): incoming gasoil curve set (krg and krog)
 
     Returns:
-        tuple of lambda functions. The first will evaluate krg on
+        tuple of functions. The first will evaluate krg on
             the normalized Sg interval [0,1], the second will
             evaluate krog on the normalized So interval [0,1].
     """
@@ -175,9 +246,12 @@ def normalize_nonlinpart_go(curve):
     # The internal dataframe might contain normalized
     # saturation values, but we do not want to assume they
     # are there or even correct, therefore we effectively
-    # recalculate them (here using lambda functions)
-    sg_fn = lambda sgn: curve.sgcr + sgn * (1.0 - curve.swl - curve.sgcr - curve.sorg)
-    krg_fn = lambda sgn: krg_interp(sg_fn(sgn))
+    # recalculate them
+    def sg_fn(sgn):
+        return curve.sgcr + sgn * (1.0 - curve.swl - curve.sgcr - curve.sorg)
+
+    def krg_fn(sgn):
+        return krg_interp(sg_fn(sgn))
 
     kro_interp = interp1d(
         1.0 - curve.table["sg"],
@@ -191,7 +265,9 @@ def normalize_nonlinpart_go(curve):
         + curve.sorg
         + son * (1.0 - curve.swl - curve.sorg - curve.sgcr)
     )
-    kro_fn = lambda son: kro_interp(so_fn(son))
+
+    def kro_fn(son):
+        return kro_interp(so_fn(son))
 
     return (krg_fn, kro_fn)
 
@@ -236,9 +312,14 @@ def normalize_pc(curve):
         bounds_error=False,
         fill_value=(max_pc, min_pc),  # This gives constant extrapolation outside [0, 1]
     )
+
     # Map from normalized value to real saturation domain:
-    sx_fn = lambda sxn: curve.table[sat_col].min() + sxn * (max_sx - min_sx)
-    pc_fn = lambda sxn: pc_interp(sx_fn(sxn))
+    def sx_fn(sxn):
+        return curve.table[sat_col].min() + sxn * (max_sx - min_sx)
+
+    def pc_fn(sxn):
+        return pc_interp(sx_fn(sxn))
+
     return pc_fn
 
 
@@ -266,7 +347,7 @@ def _interpolate_tags(low, high, parameter, tag):
             # No tags defined.
             return "Interpolated to {}".format(parameter)
         return "Interpolated to {} between {} and {}".format(
-            parameter, low.tag, high.tag,
+            parameter, low.tag, high.tag
         )
     return tag
 
@@ -309,9 +390,10 @@ def interpolate_wo(wo_low, wo_high, parameter, h=0.01, tag=None):
     pc1 = normalize_pc(wo_low)
     pc2 = normalize_pc(wo_high)
 
-    # Construct a lambda function that can be applied to both relperm values
+    # Construct a function that can be applied to both relperm values
     # and endpoints
-    weighted_value = lambda a, b: a * (1.0 - parameter) + b * parameter
+    def weighted_value(a, b):
+        return a * (1.0 - parameter) + b * parameter
 
     # Interpolate saturation endpoints
     swl_new = weighted_value(wo_low.swl, wo_high.swl)
@@ -419,7 +501,8 @@ def interpolate_go(go_low, go_high, parameter, h=0.01, tag=None):
 
     # Construct a lambda function that can be applied to both relperm values
     # and endpoints
-    weighted_value = lambda a, b: a * (1.0 - parameter) + b * parameter
+    def weighted_value(a, b):
+        return a * (1.0 - parameter) + b * parameter
 
     # Interpolate saturation endpoints
     swl_new = weighted_value(go_low.swl, go_high.swl)
