@@ -147,6 +147,11 @@ def test_make_scalrecommendation():
     assert swof.count("SATNUM X") == 1
     assert interpolant.SOF3().count("SATNUM X") == 1
 
+    # Check different comment when different interpolation parameter:
+    interpolant = rec.interpolate(-0.777, 0.888, h=0.2)
+    assert "SCAL recommendation interpolation to -0.777" in interpolant.SWOF()
+    assert "SCAL recommendation interpolation to 0.888" in interpolant.SGOF()
+
 
 def test_make_scalrecommendation_wo():
     """Test that we can make scal recommendation objects
@@ -180,8 +185,8 @@ def test_make_scalrecommendation_wo():
     interpolant = rec.interpolate(-0.5)
     check_table(interpolant.wateroil.table)
     assert interpolant.gasoil is None
-    assert sat_table_str_ok(interpolant.SWOF())
-    assert sat_table_str_ok(interpolant.SWFN())
+    sat_table_str_ok(interpolant.SWOF())
+    sat_table_str_ok(interpolant.SWFN())
 
     # This should return empty string
     assert not interpolant.SGOF()
@@ -219,8 +224,8 @@ def test_make_scalrecommendation_go():
     interpolant = rec.interpolate(-0.5)
     check_table(interpolant.gasoil.table)
     assert interpolant.wateroil is None
-    assert sat_table_str_ok(interpolant.SGOF())
-    assert sat_table_str_ok(interpolant.SGFN())
+    sat_table_str_ok(interpolant.SGOF())
+    sat_table_str_ok(interpolant.SGFN())
 
     # This should return empty string
     assert not interpolant.SWOF()
