@@ -7,6 +7,8 @@ from __future__ import print_function
 import pandas as pd
 import numpy as np
 
+import pytest
+
 from hypothesis import given, settings
 import hypothesis.strategies as st
 
@@ -15,6 +17,169 @@ from pyscal import utils, WaterOil, GasOil
 from common import check_table, float_df_checker, sat_table_str_ok
 
 
+def test_df2str():
+    """Test handling of roundoff issues when printing dataframes
+
+    See also test_gasoil.py::test_roundoff()
+    """
+    # Easy cases:
+    assert utils.df2str(pd.DataFrame(data=[0.1]), digits=1).strip() == "0.1"
+    assert utils.df2str(pd.DataFrame(data=[0.1]), digits=3).strip() == "0.100"
+    assert (
+        utils.df2str(pd.DataFrame(data=[0.1]), digits=3, roundlevel=3).strip()
+        == "0.100"
+    )
+    assert (
+        utils.df2str(pd.DataFrame(data=[0.1]), digits=3, roundlevel=4).strip()
+        == "0.100"
+    )
+    assert (
+        utils.df2str(pd.DataFrame(data=[0.1]), digits=3, roundlevel=5).strip()
+        == "0.100"
+    )
+    assert (
+        utils.df2str(pd.DataFrame(data=[0.01]), digits=3, roundlevel=2).strip()
+        == "0.010"
+    )
+
+    # Here roundlevel will ruin the result:
+    assert (
+        utils.df2str(pd.DataFrame(data=[0.01]), digits=3, roundlevel=1).strip()
+        == "0.000"
+    )
+
+    # Tricky ones:
+    # This one should be rounded down:
+    assert utils.df2str(pd.DataFrame(data=[0.0034999]), digits=3).strip() == "0.003"
+    # But if we are on the 9999 side due to representation error, the
+    # number probably represents 0.0035 so it should be rounded up
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[0.003499999999998]), digits=3, roundlevel=5
+        ).strip()
+        == "0.004"
+    )
+    # If we round to more digits than we have in IEE754, we end up truncating:
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[0.003499999999998]), digits=3, roundlevel=20
+        ).strip()
+        == "0.003"  # Wrong
+    )
+    # If we round straight to out output, we are not getting the chance to correct for
+    # the representation error:
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[0.003499999999998]), digits=3, roundlevel=3
+        ).strip()
+        == "0.003"  # Wrong
+    )
+    # So roundlevel > digits
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[0.003499999999998]), digits=3, roundlevel=4
+        ).strip()
+        == "0.004"
+    )
+    # But digits < roundlevel < 15 works:
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[0.003499999999998]), digits=3, roundlevel=14
+        ).strip()
+        == "0.004"
+    )
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[0.003499999999998]), digits=3, roundlevel=15
+        ).strip()
+        == "0.003"  # Wrong
+    )
+
+    # Double rounding is a potential issue, as:
+    assert round(0.0034445, 5) == 0.00344
+    assert round(round(0.0034445, 6), 5) == 0.00345  # Wrong
+    # So if pd.to_csv would later round instead of truncate, we could be victim
+    # of this, having roundlevel >  digits + 1 would avoid that:
+    assert round(round(0.0034445, 7), 5) == 0.00344
+    # (this is the rationale for roundlevel > digits + 1)
+
+
+def test_df2str_monotone():
+    """Test the monotonocity enforcement in df2str()"""
+
+    # Don't touch all-zero columns
+    assert (
+        utils.df2str(pd.DataFrame(data=[0, 0, 0]), digits=2, monotone_column=0)
+        == "0\n0\n0\n"
+    )
+    # A constant nonzero column, makes no sense as capillary pressure
+    # but still we ensure it runs in eclipse:
+    assert (
+        utils.df2str(pd.DataFrame(data=[1, 1, 1]), digits=2, monotone_column=0)
+        == "1.00\n0.99\n0.98\n"
+    )
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[1, 1, 1]),
+            digits=2,
+            monotone_column=0,
+            monotone_direction=-1,
+        )
+        == "1.00\n0.99\n0.98\n"
+    )
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[1, 1, 1]),
+            digits=2,
+            monotone_column=0,
+            monotone_direction="dec",
+        )
+        == "1.00\n0.99\n0.98\n"
+    )
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[1, 1, 1]),
+            digits=2,
+            monotone_column=0,
+            monotone_direction=1,
+        )
+        == "1.00\n1.01\n1.02\n"
+    )
+    assert (
+        utils.df2str(
+            pd.DataFrame(data=[1, 1, 1]),
+            digits=2,
+            monotone_column=0,
+            monotone_direction="inc",
+        )
+        == "1.00\n1.01\n1.02\n"
+    )
+    with pytest.raises(ValueError):
+        assert (
+            utils.df2str(
+                pd.DataFrame(data=[1, 1, 1]),
+                digits=2,
+                monotone_column=0,
+                monotone_direction="foo",
+            )
+            == "1.00\n1.01\n1.02\n"
+        )
+
+    assert (
+        utils.df2str(pd.DataFrame(data=[1, 1, 1]), digits=7, monotone_column=0)
+        == "1.0000000\n0.9999999\n0.9999998\n"
+    )
+
+    # Actual data that has occured:
+    dframe = pd.DataFrame(
+        data=[0.0000027, 0.0000026, 0.0000024, 0.0000024, 0.0000017], columns=["pc"]
+    )
+    assert (
+        utils.df2str(dframe, monotone_column="pc")
+        == "0.0000027\n0.0000026\n0.0000024\n0.0000023\n0.0000017\n"
+    )
+
+
 def test_diffjumppoint():
     """Test estimator for the jump in first derivative for some manually set up cases.
 
@@ -224,14 +389,14 @@ def test_tag_preservation():
     wo_high.add_corey_oil(now=3)
     interpolant1 = utils.interpolate_wo(wo_low, wo_high, parameter=0.1, h=0.2)
     assert "Interpolated to 0.1" in interpolant1.tag
-    assert sat_table_str_ok(interpolant1.SWOF())
+    sat_table_str_ok(interpolant1.SWOF())
 
     wo_high.tag = "FOOBAR"
     interpolant2 = utils.interpolate_wo(wo_low, wo_high, parameter=0.1, h=0.2)
     assert "Interpolated to 0.1" in interpolant2.tag
     assert "between" in interpolant2.tag
     assert wo_high.tag in interpolant2.tag
-    assert sat_table_str_ok(interpolant2.SWOF())
+    sat_table_str_ok(interpolant2.SWOF())
     # wo_low.tag was empty deliberately here.
 
     # When wo_log and wo_high has the same tag:
@@ -240,7 +405,7 @@ def test_tag_preservation():
     assert "Interpolated to 0.1" in interpolant3.tag
     assert "between" not in interpolant3.tag
     assert wo_high.tag in interpolant3.tag
-    assert sat_table_str_ok(interpolant3.SWOF())
+    sat_table_str_ok(interpolant3.SWOF())
 
     # Explicit tag:
     interpolant4 = utils.interpolate_wo(
@@ -254,7 +419,7 @@ def test_tag_preservation():
     )
     assert "Explicit tag" in interpolant6.tag
     print(interpolant6.SWOF())
-    assert sat_table_str_ok(interpolant6.SWOF())
+    sat_table_str_ok(interpolant6.SWOF())
 
     # Empty tag:
     interpolant5 = utils.interpolate_wo(wo_low, wo_high, parameter=0.1, h=0.2, tag="")
@@ -269,7 +434,7 @@ def test_tag_preservation():
     go_high.add_corey_oil(nog=3)
     interpolant1 = utils.interpolate_go(go_low, go_high, parameter=0.1, h=0.2)
     assert "Interpolated to 0.1" in interpolant1.tag
-    assert sat_table_str_ok(interpolant1.SGOF())
+    sat_table_str_ok(interpolant1.SGOF())
 
 
 @settings(max_examples=40, deadline=5000)
